
module ViewSupport where

import Graphics.UI.GLUT -- hiding (position)
import Graphics.Rendering.OpenGL.GLU
import Data.Maybe

--TPP
import Sprites

crMat (rd,gd,bd) (rs,gs,bs) exp = do
  materialDiffuse   Front $= Color4 (rd/255) (gd/255) (bd/255) 1.0
  materialAmbient   Front $= Color4 (rd/255) (gd/255) (bd/255) 1.0
  materialSpecular  Front $= Color4 (rs/255) (gs/255) (bs/255) 1.0
  materialShininess Front $= exp
  materialDiffuse   Back  $= Color4 (rd/255) (gd/255) (bd/255) 1.0
  materialSpecular  Back  $= Color4 (rs/255) (gs/255) (bs/255) 1.0
  materialShininess Back  $= exp

scal:: Double -> Double -> Double -> IO ()
scal = scale

tppScale :: (Double,Double,Double) -> Double -> (Double,Double,Double)
tppScale (x,y,z) s = (s*x,s*y,s*z)

transl:: Double -> Double -> Double -> IO ()
transl x y z = translate ( Vector3 x y z )

rota:: Double -> Double -> Double -> Double -> IO ()
rota a x y z = rotate a  (Vector3 x y z)

rotateZ a = rota a 0 0 1
rotateY a = rota a 0 1 0
rotateX a = rota a 1 0 0


robotContext     = crMat (196,142, 53) (52,52,52) 10
shellContext     = crMat (112,150,186) (1,1,1) 0

-- Draw Small Reference Axes
redContext       = crMat (255,  0,  0) (102,102,102) 10
greenContext     = crMat (  0,255,  0) (102,102,102) 10
blueContext      = crMat (  0,  0,255) (102,102,102) 10
greyContext      = crMat (150,150,150) (102,102,102) 10
whiteContext     = crMat (255,255,255) (102,102,102) 10
blackContext     = crMat (  0,  0,  0) (102,102,102) 10
xBeamContext     = crMat (148, 79,  1) (102,102,102) 10
sBeamContext     = crMat (100, 64, 44) (102,102,102) 5
fBeamContext     = crMat (205, 101,45) (10,10,10) 5

referenceAxes = do

  preservingMatrix $ do

    redContext
    preservingMatrix $ do
      rotateY (90)
      renderObject Solid (Cylinder' 0.04 0.5 8 1)

    greenContext
    preservingMatrix $ do
      rotateX (-90)
      renderObject Solid (Cylinder' 0.04 0.5 8 1)

    blueContext
    preservingMatrix $ do
      renderObject Solid (Cylinder' 0.04 0.5 8 1)

-----------------------------------------------------------------------------

polygon :: (Double,Double,Double) -> (Double,Double,Double) ->
            (Double,Double,Double) -> (Double,Double,Double) -> 
                       (Double,Double,Double) -> Maybe TextureObject -> [TexCoord2 Double] -> IO () 
polygon q0 q1 q2 q3 (nx,ny,nz) mtex@(Just tex) ts = do
    preservingMatrix $ do
       --greyContext
       textureBinding Texture2D $= mtex  
       renderPrimitive Quads $ do
         normal (Normal3 nx ny nz)
         mapM_ (\((vx, vy, vz),tcoord) -> do
                  texCoord tcoord
                  vertex $ Vertex3 vx vy vz) 
                      (zip ([q0]++[q1]++[q2]++[q3]) ts)

polygon q0 q1 q2 q3 (nx,ny,nz) Nothing _ = do
    preservingMatrix $ do
       --greyContext
       renderPrimitive Quads $ do
         normal (Normal3 nx ny nz)
         mapM_ (\(vx, vy, vz) -> 
                  vertex $ Vertex3 vx vy vz) 
                      ([q0]++[q1]++[q2]++[q3])

cubeoidTex :: [Maybe TextureObject] ->  Double -> Double -> Double -> IO ()
cubeoidTex texs a b c = do

  preservingMatrix $ do
    textureFunction $= Modulate
    polygon v0 v3 v2 v1 n0 (texs!!0) ts -- front face (with z towards observer, x left, and y up)
    polygon v2 v3 v7 v6 n1 (texs!!1) ts -- right face
    polygon v3 v0 v4 v7 n2 (texs!!2) ts -- bottom face
    polygon v1 v2 v6 v5 n3 (texs!!3) ts -- top face
    polygon v4 v5 v6 v7 n4 (texs!!4) ts -- rear face
    polygon v5 v4 v0 v1 n5 (texs!!5) ts -- left face

  where 
    v0 = ((-a/2),(-b/2),( c/2)) -- 0
    v1 = ((-a/2),( b/2),( c/2)) -- 1
    v2 = (( a/2),( b/2),( c/2)) -- 2
    v3 = (( a/2),(-b/2),( c/2)) -- 3
    v4 = ((-a/2),(-b/2),(-c/2)) -- 4
    v5 = ((-a/2),( b/2),(-c/2)) -- 5
    v6 = (( a/2),( b/2),(-c/2)) -- 6
    v7 = (( a/2),(-b/2),(-c/2)) -- 7

    n0 = (  0,  0,  1)
    n1 = (  1,  0,  0)
    n2 = (  0,(-1), 0)
    n3 = (  0,  1,  0)
    n4 = (  0,  0,(-1))
    n5 = ((-1), 0,  0)

    ts = [ TexCoord2 0 1
         , TexCoord2 1 1 
         , TexCoord2 1 0
         , TexCoord2 0 0
         ]

mapVertices :: [(TexCoord2 Double)] -> [(Vertex3 Double)] -> IO ()
mapVertices texs verts = mapM_ setVertex (zip texs verts)

setVertex :: (TexCoord2 Double, Vertex3 Double) -> IO ()
setVertex (texCoordinates, vertexCoordinates) = do 
                    texCoord texCoordinates; vertex vertexCoordinates;

